import requests
import json
from pprint import pprint
import pandas as pd
from pptx import Presentation
from pptx.dml.color import RGBColor
from pptx.enum.text import PP_ALIGN  # テキストフレームの位置調整のためのクラス
from pptx.util import Pt             # ポイント指定するためのクラス
import openpyxl
import sqlite3


class KintoneClass:
    def __init__(self):
        self.config = None

    def func1(self):
        pprint(self.config)

    def GetAllRecords(self):
        records = self.data_select()
        with open('temp/records.json', 'w', encoding='utf-8') as file:
            json.dump(records, file, ensure_ascii=False, indent=2)

        output_data = []
        for entry in records:
            new_entry = {}
            for key, value in entry.items():
                new_entry[key] = value["value"]
            output_data.append(new_entry)

        with open('temp/records2.json', 'w', encoding='utf-8') as file:
            json.dump(output_data, file, ensure_ascii=False, indent=2)

        # output_dataをto_sqlでsqlite3のテーブルに格納する
        conn = sqlite3.connect('temp/records.db')
        df = pd.DataFrame(output_data)
        df.to_sql(f'tbl{self.config["app_id"]}',
                  conn, if_exists='replace', index=False)

    def data_select(self, offset=0):
        """
        kintoneのレコードを取得(再帰的に呼び出す)
        """

        limit = 500
        # domainは適切な値に置き換える必要があります
        url = "https://{}.cybozu.com/k/v1/records.json".format(
            self.config["domain"])
        headers = {"X-Cybozu-API-Token": self.config["api_token"],
                   "Content-Type": "application/json"}

        fields = ["レコード番号", "製品番号", "製品名", "予定1",
                  "実績1", "予定2", "実績2", "予定3", "実績3", "処理日"]

        params = {
            "app": self.config["app_id"],
            "fields": fields,
            "query": f"order by $id limit {limit} offset {offset}"
        }

        resp = requests.get(url, json=params, headers=headers)
        data = resp.json()
        records = data["records"]

        # もし取得したレコード数がlimitと等しい場合は再帰的に呼び出して残りのレコードを取得
        if len(records) == limit:
            next_records = self.data_select(offset + limit)
            records.extend(next_records)

        return records

    def loadConfig(self):
        with open('temp/selected_conf.json', 'r', encoding='utf-8') as file:
            self.config = json.load(file)

    def calculate(self, debug=False, calc_mode='excel'):

        with open('temp/selected_conf.json', 'r', encoding='utf-8') as file:
            param = json.load(file)

        with open('temp/records.json', 'r', encoding='utf-8') as file:
            result = json.load(file)

        list_out = []
        for input_data in result:
            output_data = {}
            for key, value in input_data.items():
                if 'value' in value:
                    if key in ['作成者', '更新者']:
                        output_data[key] = value['value']['name']
                    else:
                        output_data[key] = value['value']
                else:
                    output_data[key] = value

            list_out.append(output_data)
        df = pd.DataFrame(list_out)
        o = {}
        if calc_mode == 'excel':
            cmd = param['算出式']
        else:
            with open(f'temp/command.txt', 'r', encoding='utf-8') as file:
                cmd = file.read().strip()
        exec(cmd)

        with open('temp/ppt_data.json', 'w', encoding='utf-8') as file:
            json.dump(o, file, ensure_ascii=False, indent=4)

        if debug == True:
            pprint(o)
        else:
            print(f'{param["設定名"]} の計算を実行しました')

        return o

    def createPptx(self, debug=False):

        with open('temp/selected_conf.json', 'r', encoding='utf-8') as file:
            param = json.load(file)

        template = f"template/{param['pptxテンプレート']}"
        output_file = f"output/{param['出力pptx名']}"

        with open('temp/ppt_data.json', 'r', encoding='utf-8') as file:
            dictOut = json.load(file)

        prs = Presentation(template)  # 既存のファイルを読み込む
        table = prs.slides[0].shapes[1].table    # Tableオブジェクトの取得

        for i in range(len(table.columns)):	        # テーブルの列数の取得
            for j in range(len(table.rows)):   # テーブルの行数の取得
                cell = table.cell(j, i)
                txt = cell.text

                for key, value in dictOut.items():
                    if txt == key:
                        cell.text = str(value)

        prs.save(f'{output_file}.pptx')

        if debug == True:
            print(f'{output_file}.pptx に出力しました')
        else:
            print(f'{param["設定名"]} のpptxを作成しました')

    def readConfig(self, debug=False):
        wb = openpyxl.load_workbook(f'conf/kintone2pptx_設定.xlsx')
        sheet = wb.worksheets[0]
        data_dict = {}
        for row in sheet.iter_rows(values_only=True):
            key = row[0]
            value = row[1]
            if key != '項目':
                # value = int(value)
                data_dict[key] = value
        # list_out.append(data_dict)
        with open('temp/selected_conf.json', 'w', encoding='utf-8') as file:
            json.dump(data_dict, file, ensure_ascii=False, indent=4)

    def getSql(self, debug=False):

        with open('temp/sql.json', 'r', encoding='utf-8') as file:
            listSQL = json.load(file)

        conn = sqlite3.connect('temp/records.db')
        cursor = conn.cursor()
        dictOut = {}
        for item in listSQL:
            key = item[0]
            sql = item[1]
            cursor.execute(sql)
            count = cursor.fetchone()[0]
            item.append(count)
            dictOut[key] = count
        cursor.execute(sql)
        count = cursor.fetchone()[0]
        conn.close()

        with open('temp/ppt_data.json', 'w', encoding='utf-8') as file:
            json.dump(dictOut, file, ensure_ascii=False, indent=4)

        if debug == True:
            return dictOut


if __name__ == "__main__":

    kintone = KintoneClass()
    # kintone.readConfig(debug=True)
    kintone.loadConfig()
    # kintone.func1()
    # kintone.GetAllRecords()

    # listSQL = [
    #     ['A1', 'select count(*) from tbl where 実績1 is not null'],
    #     ['B1', 'select count(*) from tbl where 実績2 is not null'],
    #     ['C1', 'select count(*) from tbl where 実績3 is not null'],
    # ]

    # with open('temp/sql.json', 'w', encoding='utf-8') as file:
    #     json.dump(listSQL, file, ensure_ascii=False, indent=4)

    ret = kintone.getSql(debug=True)
    pprint(ret)
    # kintone.calculate(debug=True, calc_mode='text')
    # kintone.createPptx(debug=True)
