from pptx import Presentation
from pptx.dml.color import RGBColor
from pptx.enum.text import PP_ALIGN  # テキストフレームの位置調整のためのクラス
from pptx.util import Pt             # ポイント指定するためのクラス
# import io
import requests
from pprint import pprint
import json
import pandas as pd
import openpyxl
import sys
import os
import logging
import yaml


fname = os.path.abspath(__file__)
root1 = os.path.dirname(fname)
root = os.path.dirname(root1)
# print(root)
# print(f'{root}/conf/kintone2pptx_設定.xlsx')

# sys.exit()


logging.basicConfig(level=logging.INFO, filename=f"{root}/log/log.txt")
# logging.info("This is an INFO level log.")


CONF_FILE = f'{root}/temp/conf.json'
SELECTED_CONF_FILE = f'{root}/temp/selected_conf.json'
DATA_FILE = f'{root}/temp/temp.json'
PPT_DATA = f'{root}/temp/ppt_data.json'


def createPptx(debug=False):

    with open(SELECTED_CONF_FILE, 'r', encoding='utf-8') as file:
        param = json.load(file)

    template = f"{root}/template/{param['pptxテンプレート']}"
    output_file = f"{root}/output/{param['出力pptx名']}"

    with open(PPT_DATA, 'r', encoding='utf-8') as file:
        dictOut = json.load(file)

    prs = Presentation(template)  # 既存のファイルを読み込む
    table = prs.slides[0].shapes[1].table    # Tableオブジェクトの取得

    for i in range(len(table.columns)):	        # テーブルの列数の取得
        for j in range(len(table.rows)):   # テーブルの行数の取得
            cell = table.cell(j, i)
            txt = cell.text

            for key, value in dictOut.items():
                if txt == key:
                    cell.text = str(value)

    prs.save(f'{output_file}.pptx')

    if debug == True:
        print(f'{output_file}.pptx に出力しました')
    else:
        print(f'{param["設定名"]} のpptxを作成しました')


def readKintone(debug=False):
    """
    kintoneのレコードを複数取得
    """
    with open(SELECTED_CONF_FILE, 'r', encoding='utf-8') as file:
        param = json.load(file)

    url = f"https://{param['domain']}.cybozu.com/k/v1/records.json"
    headers = {"X-Cybozu-API-Token": param['api_token'],
               "Content-Type": "application/json"}

    params = {
        "app": param['app_id'],
    }

    resp = requests.get(url, json=params, headers=headers)
    result = resp.json()
    with open(DATA_FILE, 'w', encoding='utf-8') as file:
        json.dump(result, file, ensure_ascii=False, indent=4)

    if debug == True:
        pprint(result)
    else:
        print(f'{param["設定名"]} のレコードを取得しました')


def calculate(debug=False, calc_mode='excel'):

    with open(SELECTED_CONF_FILE, 'r', encoding='utf-8') as file:
        param = json.load(file)

    with open(DATA_FILE, 'r', encoding='utf-8') as file:
        result = json.load(file)

    list_out = []
    for input_data in result['records']:
        output_data = {}
        for key, value in input_data.items():
            if 'value' in value:
                if key in ['作成者', '更新者']:
                    output_data[key] = value['value']['name']
                else:
                    output_data[key] = value['value']
            else:
                output_data[key] = value

        list_out.append(output_data)
    df = pd.DataFrame(list_out)
    o = {}
    if calc_mode == 'excel':
        cmd = param['算出式']
    else:
        with open(f'{root}/temp/command.txt', 'r', encoding='utf-8') as file:
            cmd = file.read().strip()
    exec(cmd)

    with open(PPT_DATA, 'w', encoding='utf-8') as file:
        json.dump(o, file, ensure_ascii=False, indent=4)

    if debug == True:
        pprint(o)
    else:
        print(f'{param["設定名"]} の計算を実行しました')


def readConfig(debug=False):
    wb = openpyxl.load_workbook(f'{root}/conf/kintone2pptx_設定.xlsx')

    dictConfig = {}
    for sheet in wb:
        data_dict = {}
        for row in sheet.iter_rows(values_only=True):
            key = row[0]
            value = row[1]
            if key != '項目':
                # value = int(value)
                data_dict[key] = value
        # list_out.append(data_dict)
        dictConfig[data_dict['設定名']] = data_dict

    with open(CONF_FILE, 'w', encoding='utf-8') as file:
        json.dump(dictConfig, file, ensure_ascii=False, indent=4)

    if debug:
        pprint(dictConfig)

    return dictConfig


if __name__ == '__main__':

    print('----- 処理開始 -----')
    logging.info('----- 処理開始log -----')

    if len(sys.argv) > 1 and sys.argv[1] == 'calculate':
        calculate(True, calc_mode='text')
    elif len(sys.argv) > 1 and sys.argv[1] == 'readConfig':
        listParam = readConfig(debug=True)
    elif len(sys.argv) > 1 and sys.argv[1] == 'readKintone':
        listParam = readConfig()
        readKintone(True)
    elif len(sys.argv) > 1 and sys.argv[1] == 'createPptx':
        createPptx(True)
    else:
        listParam = readConfig()
        for key, param in listParam.items():
            # with open(SELECTED_CONF_FILE, 'r', encoding='utf-8') as file:
            #     param = json.load(file)
            with open(SELECTED_CONF_FILE, 'w', encoding='utf-8') as file:
                json.dump(param, file, ensure_ascii=False, indent=4)
            readKintone()
            calculate(debug=False, calc_mode='excel')
            createPptx()

    logging.info('----- 完了log -----')
    print('----- 処理完了 -----')
